/***************************************************************************
                          kmmsdock.cpp  -  description
                             -------------------
    begin                : Mon Mar  5 16:18:00 CET 2001
    copyright            : (C) 2001 by Janosch Rolles
    email                : janosch@base2.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmmsdock.h"



/* 
 *  Constructs a kmmsdocks a child of 'parent', with the
 *  name 'name' and widget flags set to 'fl
 */
kmmsdock::kmmsdock( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
  if ( !name )
		setName( "kmmsdock" );

	setAcceptDrops(TRUE);

  setCaption( i18n( "KmmsDockApplet" ) );
}

void kmmsdock::prefsPopup()
{
  kmmsprefsImpl kp(this, "Kmms Preferences", TRUE);
  if ( kp.getPrefs( &titleColor, &osdColor, &clearPlaylistOnDrop, &autoPlayAddedSongs ) )
		savePreferences();
}

void kmmsdock::loadPreferences()
{
  kmmsConfig = new KSimpleConfig(homeDir + "/.kmms/kmmsrc");

	// COLORS
  kmmsConfig->setGroup("COLORS");

	//titleColor = QColor(32, 153, 255);
  titleColor 	= QColor(	kmmsConfig->readEntry( "titleColorR", "32" ).toInt(),
												kmmsConfig->readEntry( "titleColorG", "153" ).toInt(),
												kmmsConfig->readEntry( "titleColorB", "255" ).toInt()		);

	//osdColor = QColor(0, 245, 0);
  osdColor 		= QColor(	kmmsConfig->readEntry( "osdColorR", "0" ).toInt(),
												kmmsConfig->readEntry( "osdColorG", "245" ).toInt(),
												kmmsConfig->readEntry( "osdColorB", "0" ).toInt()			);

	// PLAYLIST
	kmmsConfig->setGroup("PLAYLIST");	

	// clearPlaylistOnDrop = false;
	if ( kmmsConfig->readEntry( "clearPlaylistOnDrop", "false" ) == "true" )
		clearPlaylistOnDrop = true;
	else
		clearPlaylistOnDrop = false;

	// autoPlayAddedSongs = true;
	if ( kmmsConfig->readEntry( "autoPlayAddedSongs", "true" ) == "true" )
		autoPlayAddedSongs = true;
	else
		autoPlayAddedSongs = false;
}

void kmmsdock::savePreferences()
{
	// COLORS
  kmmsConfig->setGroup("COLORS");

	kmmsConfig->writeEntry( "titleColorR", QString::number( titleColor.red() ) );
	kmmsConfig->writeEntry( "titleColorG", QString::number( titleColor.green() ) );
	kmmsConfig->writeEntry( "titleColorB", QString::number( titleColor.blue() ) );

	kmmsConfig->writeEntry( "osdColorR", QString::number( osdColor.red() ) );
	kmmsConfig->writeEntry( "osdColorG", QString::number( osdColor.green() ) );
	kmmsConfig->writeEntry( "osdColorB", QString::number( osdColor.blue() ) );

	// PLAYLIST
	kmmsConfig->setGroup("PLAYLIST");	

	if ( clearPlaylistOnDrop )
		kmmsConfig->writeEntry( "clearPlaylistOnDrop", "true" );
	else
		kmmsConfig->writeEntry( "clearPlaylistOnDrop", "false" );

	if ( autoPlayAddedSongs )
		kmmsConfig->writeEntry( "autoPlayAddedSongs", "true" );
	else
		kmmsConfig->writeEntry( "autoPlayAddedSongs", "false" );

	// save values
	kmmsConfig->sync();
}

void kmmsdock::initialize(const char* version)
{
  /* get homedir */
	homeDir = QDir::home().path();

	/* Preferences */
	loadPreferences();
	savePreferences();

	kmmsWidth = KMMSDOCK_WIDTH;
  kmmsFullHeight = KMMSDOCK_FULLHEIGHT;

	// initialize (so we can't get crashes)
	//  -> will be reinitialized by widthForHeight() / heightForWidth()
	if (!kmmsCurrentHeight)
		kmmsCurrentHeight = KMMSDOCK_FULLHEIGHT;

  //gettitledelay = 0;
  xPos = kmmsWidth;
  scrollStep = 1;

	scrollDelay = SCROLL_DELAY_NS;
	receiveDelay = RECEIVE_DELAY_NS;

  welcomeDisplayed = false;

	displayVolume = 0;

	pmt = QPixmap::QPixmap( kmmsWidth, kmmsFullHeight );

	titleFont = new QFont("helvetica",10,QFont::Normal);

	// welcome message: first time song display
  title = "Welcome to KMMS (K Multi Media System) version " + QString::fromLatin1(version);
	QFontMetrics titleFm( *titleFont );
	titleWidth = titleFm.width( title );

	// Fisrt time paint
  makeSkinBuffer(); // needed by paintTitle()
  paint(); // first time - repainted by scroll()

	// refresh by timer
  QTimer *scrollTimer = new QTimer(this);
  connect( scrollTimer, SIGNAL(timeout()), SLOT( scroll() ) );
  scrollTimer->start( scrollDelay );			

	// here we fetch info from xmms
  QTimer *receiveTimer = new QTimer(this);
  connect( receiveTimer, SIGNAL(timeout()), SLOT( receive() ) );
  receiveTimer->start( receiveDelay );
}

int kmmsdock::widthForHeight( int height )
{
	kmmsCurrentHeight = height;
	kmmsOrientation = HORIZ;
	resize(KMMSDOCK_WIDTH, height);
	return KMMSDOCK_WIDTH;
}

int kmmsdock::heightForWidth( int width )
{
	kmmsCurrentHeight = width;
	kmmsOrientation = VERTI;
	resize(width, KMMSDOCK_WIDTH);
	return KMMSDOCK_WIDTH; // vertical (flip 90)
}

void kmmsdock::makeSkinBuffer()
{
  QPixmap skin( kmmsWidth, kmmsFullHeight );
  //skin.fill( this, kmmsWidth, kmmsFullHeight );
  QPainter p;
  p.begin( &skin );
    paintSkin( &p );
  p.end();
  skinpm = QPixmap(skin);
  skinpm.setDefaultOptimization( QPixmap::BestOptim ); // good for very often used pm
}

void  kmmsdock::paintSkin( QPainter *pskin ) // called from makeSkinBuffer()
{
  //load and paint Skin
  QPixmap mainPixmap;
//  QPixmap cbuttonsPixmap;
  bool skinloadfailed = FALSE;
  // main.bmp
  if ( !mainPixmap.load( homeDir + "/.kmms/Skin/main.bmp") )
   	skinloadfailed = TRUE;
  // cbuttons.bmp
  if ( !cbuttonsPixmap.load( homeDir + "/.kmms/Skin/cbuttons.bmp") )
    skinloadfailed = TRUE;
	// failed?
  if(skinloadfailed)
  {
		// draw error message to skin buffer
    pskin->drawText( 2, 12, "SKIN LOAD ERROR!" );
    pskin->drawText( 8, 28, "( ~/.kmms/Skin )" );
    pskin->drawText( 8, 40, "view README file" );
  }
	// success?
  else
  {
		// draw skin to buffer
		/* //old version
    pskin->drawPixmap( -135, 0, mainPixmap );
    pskin->drawPixmap( 0, 36, cbuttonsPixmap );
		*/
		// new version
		
		// Top:
    pskin->drawPixmap( 0, 0, mainPixmap, 120, 15, KMMSDOCK_WIDTH, 5 );
		pskin->drawPixmap( 0, 5, mainPixmap, 120, 15, KMMSDOCK_WIDTH, 8 );

		// Top of Text field
		pskin->drawPixmap( 0, 13, mainPixmap, 120, 23, KMMSDOCK_WIDTH, 6 );

		// Text field extension
		pskin->drawPixmap( 0, 19, mainPixmap, 120, 29, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 20, mainPixmap, 120, 29, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 21, mainPixmap, 120, 29, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 22, mainPixmap, 120, 29, KMMSDOCK_WIDTH, 1 );

		pskin->drawPixmap( 0, 23, mainPixmap, 120, 30, KMMSDOCK_WIDTH, 1 );
			//pskin->drawPixmap( 0, 23, mainPixmap, 120, 30, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 24, mainPixmap, 120, 30, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 25, mainPixmap, 120, 30, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 26, mainPixmap, 120, 30, KMMSDOCK_WIDTH, 1 );

		pskin->drawPixmap( 0, 27, mainPixmap, 120, 31, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 28, mainPixmap, 120, 31, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 29, mainPixmap, 120, 31, KMMSDOCK_WIDTH, 1 );
			pskin->drawPixmap( 0, 30, mainPixmap, 120, 31, KMMSDOCK_WIDTH, 1 );

		// Bottom of Text field
		pskin->drawPixmap( 0, 31, mainPixmap, 120, 32 );

		// Control Buttons
    pskin->drawPixmap( 0, 36, cbuttonsPixmap, 0, 0, 114, 18); //painting just the top part of the c buttons
  }
}

void kmmsdock::scroll()
{
  xPos = (xPos - scrollStep);
  if( xPos < -titleWidth )
  {
    welcomeDisplayed = true;
    updateTitle();
    xPos = kmmsWidth;
    scrollStep = 1;
    scrollDelay = SCROLL_DELAY_NS;
  }
  paint();
}

void kmmsdock::paint()
{
  pt.begin( &pmt );
    // skin
    pt.drawPixmap( 0, 0, skinpm );
    // title
    pt.setFont( *titleFont );
    pt.setPen( titleColor );
    // old version
		//pt.drawText( xPos, 35, title );
		// new version
		pt.drawText( xPos, 29, title );
		if (displayVolume != 0)
		{
			// add own color + prefs
      pt.fillRect(7, (kmmsFullHeight - kmmsCurrentHeight) + 3, volume, 6, osdColor );
		}
  pt.end();
	hvpainter.begin( this );
	  if ( kmmsOrientation == VERTI )
		{
			hvpainter.saveWorldMatrix();
    	hvpainter.translate( 0 , height() );
			hvpainter.rotate( -90 );
			hvpainter.drawPixmap( 0, width() - kmmsFullHeight, pmt );
			hvpainter.restoreWorldMatrix();
  	}
		else if ( kmmsOrientation == HORIZ )
		{
			hvpainter.drawPixmap( 0, kmmsCurrentHeight - kmmsFullHeight, pmt );
  	}
	hvpainter.end();
}

void kmmsdock::updateTitle()
{
  title = temptitle;
	QFontMetrics titleFm( *titleFont );
	titleWidth = titleFm.width( title );
}

/*
 * get values from XMMS
 *
 */
void kmmsdock::receive() {
  lasttitle 			= temptitle;
	lastvolume			= volume;

  playlistpos     = xmms_remote_get_playlist_pos(0);
  volume          = xmms_remote_get_main_volume(0); if (volume > 100 || volume < 0) volume = 0;
  currentTime     = xmms_remote_get_output_time(0);
  trackTime       = xmms_remote_get_playlist_time(0, playlistpos);
  temptitle       = xmms_remote_get_playlist_title(0, playlistpos);

	// speedup scrolling when title has changed
  if( qstrcmp(temptitle, lasttitle) != 0 && welcomeDisplayed ) {
	  speedup();
  }

	// activate volume display if it has changed
	if (lastvolume != volume)
		displayVolume = 10;
	else
		if (displayVolume > 0)
			displayVolume--;

  if ((trackTime == 0) && (playlistpos == 0)) {
    xmmsStarted = false;
  } else {
    xmmsStarted = true;
  }
}

void kmmsdock::speedup()
{
  scrollStep = 1 + ( xPos + titleWidth ) / 24;
}

void kmmsdock::ButtonPressed(int x) {
  QPainter p;
  p.begin( &skinpm );
  if (x==1) {
    p.drawPixmap( 0, 36, cbuttonsPixmap, 0, 18, 23, 18); //painting just the first pressed of the c buttons
  }
  if (x==2) {
    p.drawPixmap( 23, 36, cbuttonsPixmap, 23, 18, 23, 18); //painting just the second pressed of the c buttons
  }
  if (x==3) {
    p.drawPixmap( 46, 36, cbuttonsPixmap, 46, 18, 23, 18); //painting just the third pressed of the c buttons
  }
  if (x==4) {
    p.drawPixmap( 69, 36, cbuttonsPixmap, 69, 18, 23, 18); //painting just the fourth pressed of the c buttons
  }
  if (x==5) {
    p.drawPixmap( 92, 36, cbuttonsPixmap, 92, 18, 23, 18); //painting just the fifth pressed of the c buttons
  }
  p.end();
  skinpm.setDefaultOptimization( QPixmap::BestOptim ); // good for very often used pm
}

void kmmsdock::ButtonReleased() {
  QPainter p;
  p.begin( &skinpm );
    p.drawPixmap( 0, 36, cbuttonsPixmap, 0, 0, 114, 18); //painting just the top part of the c buttons
  p.end();
  skinpm.setDefaultOptimization( QPixmap::BestOptim ); // good for very often used pm
}

/*
 * drag and drop support:
 * dragged object enters xmms-kde
 *
 */
void kmmsdock::dragEnterEvent(QDragEnterEvent* event) {
  event->accept(QTextDrag::canDecode(event));
}

/*
 * drag and drop support:
 * dragged object is dropped onto kmms
 *
 */
void kmmsdock::dropEvent(QDropEvent* event) {
  QString text;
  if (QTextDrag::decode(event, text)) {
    /*
     * test if XMMS is currently playing and stop
     */
    gboolean playing = xmms_remote_is_playing(0);
    /*
     * get length of playlist (to start playing added songs later)
     */
		int playlistlength = xmms_remote_get_playlist_length(0);
    /*
     * clear the playlist
     */
		if (clearPlaylistOnDrop)
			xmms_remote_playlist_clear(0);
    /*
     * add the file that was dropped
     */
    xmms_remote_playlist_add_url_string(0, (gchar *) text.latin1());
    /*
     * if XMMS was playing before, restart playing after playlist clear
     */
    if (playing && clearPlaylistOnDrop)
			xmms_remote_play(0);
    /*
     * Play just added song(s)
     */
		if (autoPlayAddedSongs)
		{
			//xmms_remote_set_playlist_pos(0, playlistlength + 1);
			// + 1 didn't work - too fast ??
			xmms_remote_set_playlist_pos(0, playlistlength);
			if (!playing) xmms_remote_play(0);
		}
  }
}

/*
 * the remote control buttons
 * and other mouse functions
 *
 */
void kmmsdock::mouseReleaseEvent(QMouseEvent *event)
{
  ButtonReleased();
  // left mouse button: remote control buttons
  if (event->button() == 1)
  {
		if (!xmmsStarted) {
    	xmmsStart();
			return;
  	}
		// do mapping for vertical mode
		int x = -1;
		int y = -1;
    if ( kmmsOrientation == HORIZ )
    {
			x = event->x();
			y = event->y();
		}
		else if ( kmmsOrientation == VERTI )
		{
			x = kmmsWidth - event->y();
			y = event->x();
		}
		// mouse position over button?
    if ( y + (kmmsFullHeight - kmmsCurrentHeight) >= 42) // 42 is top of buttons painted by paintSkin
    {
      if ( x >= 23*(1-1) && x < 23*(1-0) )
				xmmsBack();
      if ( x >= 23*(2-1) && x < 23*(2-0) )
				xmmsPlay();
      if ( x >= 23*(3-1) && x < 23*(3-0) )
				xmmsPause();
      if ( x >= 23*(4-1) && x < 23*(4-0))
				xmmsStop();
      if ( x >= 23*(5-1) && x < 23*(5-0))
				xmmsSkip();
    }
  }
  // middle mouse button: pause on whole remote
  if (event->button() == 4) // button 4: middle mouse button
  {
    xmmsPause();
  }
  // right mouse button: popup menu on whole remote
  if (event->button() == 2)
  {
    // not implemented yet
    //QRect g = KWM::geometry( this->winId() );
    //menu->popup(QPoint( g.x() + g.width(), g.y() + g.height()));
  }
}

void kmmsdock::mouseMoveEvent(QMouseEvent *event) {
//if (event->button() == QMouseEvent::LeftButton) {
		int x = -1;
		int y = -1;
    if ( kmmsOrientation == HORIZ )
    {
			x = event->x();
			y = event->y();
		}
		else if ( kmmsOrientation == VERTI )
		{
			x = kmmsWidth - event->y();
			y = event->x();
		}
		// mouse position over button?
    ButtonReleased();
    if ( y + (kmmsFullHeight - kmmsCurrentHeight) >= 42) // 42 is top of buttons painted by paintSkin
    {
      if ( x >= 23*(1-1) && x < 23*(1-0) )
				ButtonPressed(1);
      if ( x >= 23*(2-1) && x < 23*(2-0) )
				ButtonPressed(2);
      if ( x >= 23*(3-1) && x < 23*(3-0) )
				ButtonPressed(3);
      if ( x >= 23*(4-1) && x < 23*(4-0))
				ButtonPressed(4);
      if ( x >= 23*(5-1) && x < 23*(5-0))
				ButtonPressed(5);
    }
}

void kmmsdock::mousePressEvent(QMouseEvent *event) {
if (event->button() == 1) {
		int x = -1;
		int y = -1;
    if ( kmmsOrientation == HORIZ )
    {
			x = event->x();
			y = event->y();
		}
		else if ( kmmsOrientation == VERTI )
		{
			x = kmmsWidth - event->y();
			y = event->x();
		}
		// mouse position over button?
    if ( y + (kmmsFullHeight - kmmsCurrentHeight) >= 42) // 42 is top of buttons painted by paintSkin
    {
      if ( x >= 23*(1-1) && x < 23*(1-0) )
				ButtonPressed(1);
      if ( x >= 23*(2-1) && x < 23*(2-0) )
				ButtonPressed(2);
      if ( x >= 23*(3-1) && x < 23*(3-0) )
				ButtonPressed(3);
      if ( x >= 23*(4-1) && x < 23*(4-0))
				ButtonPressed(4);
      if ( x >= 23*(5-1) && x < 23*(5-0))
				ButtonPressed(5);
    }
  }
}

/*
 * mouse wheel -> volume
 * ( delta() usually is +/- 120 )
 *
 */
void kmmsdock::wheelEvent(QWheelEvent *event)
{
	volume = volume + event->delta() / 30;
	if (volume > 100)
		volume = 100;
	if (volume < 0)
		volume = 0;
	xmms_remote_set_main_volume(0, volume);
	displayVolume = 10;
}

void kmmsdock::xmmsStart()
{
	KShellProcess proc;
  proc << "xmms -m";
  proc.start(KShellProcess::DontCare);
}

void kmmsdock::xmmsBack()
{
	xmms_remote_playlist_prev(0);	
}

void kmmsdock::xmmsPlay()
{
	xmms_remote_play(0);
}

void kmmsdock::xmmsPause()
{
	xmms_remote_pause(0);
}

void kmmsdock::xmmsStop()
{
	xmms_remote_stop(0);
}

void kmmsdock::xmmsSkip()
{
	xmms_remote_playlist_next(0);
}

void kmmsdock::xmmsVolup()
{
	volume = volume + 4;
	if (volume > 100)
		volume = 100;
	xmms_remote_set_main_volume(0, volume);
	displayVolume = 10;
}

void kmmsdock::xmmsVoldw()
{
	volume = volume - 4;
	if (volume < 0)
		volume = 0;
	xmms_remote_set_main_volume(0, volume);
	displayVolume = 10;
}


/*  
 *  Destroys the object and frees any allocated resources
 */
kmmsdock::~kmmsdock()
{
    // no need to delete child widgets, Qt does it all for us
}

#include "kmmsdock.moc"

